"""Support for loading OpenDRIVE maps.

.. raw:: html

   <h2>Submodules</h2>

.. autosummary::
   :toctree:

   workspace
   xodr_parser
"""

from .workspace import OpenDriveWorkspace
from .xodr_parser import OpenDriveWarning
